--[[*************************************************************
File: Core.lua          AddOn: SLClock
Rev Date: 12/28/2007    Author: Taffu

SLClock is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SLClock is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SLClock.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************]]

local L = AceLibrary("AceLocale-2.2"):new("SLClock")
local SML = AceLibrary("LibSharedMedia-2.0")

SLClock = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "SLModule-1.0")
SLClock:RegisterDB("SLClockVar")

local db, text
local self = SLClock

function SLClock:OnInitialize()
    self:RegisterDefaults("profile", self.defaults)
    self:RegisterChatCommand("/slclock", self.opts)
    db = self.db.profile
    -- Register this module
    if IsAddOnLoaded("SLCore") then
        if self:IsSelfActive(self) then return else
            self:RegisterSLModule(SLClock, "SLClock")
        end
    else error("SLClock requires SLCore") end
    -- Create frame
    self:CreateFrame()
end

function SLClock:OnEnable()
    self:ScheduleRepeatingEvent(self.UpdateMod, db.intDelay, self)
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "Update")
    self:BuildOptions()
    self.frame:Show()
end

function SLClock:OnDisable()
    self.frame:Hide()
end

function SLClock:Update()
    self:UpdateFrame(self)
    self:UpdateMod()
end

function SLClock:UpdateMod()
    local text1, text2
    local color1, color2
    if db.isMasterColors then
        color1 = self:GetColorValue(SLCore.db.profile.masterColor1)
        color2 = self:GetColorValue(SLCore.db.profile.masterColor2)
    else
        color1 = self:GetColorValue(db.color1)
        color2 = self:GetColorValue(db.color2)
    end
    
    local hour, minute, seconds, clockSep = date("%H"), date("%M"), date("%S"), db.seperator
    if db.is24Hour then
        if db.showSeconds then
            text1 = string.format("%s%s%s%s%s", hour, clockSep, minute, clockSep, seconds)
        else
            text1 = string.format("%s%s%s", hour, clockSep, minute)
        end
        text2 = ""
    else
        local fixHour = math.floor(hour-hour)+hour
        if (fixHour > 11 and fixHour ~= 24) then
            if fixHour > 12 then newHour = math.floor(fixHour-12) else newHour = fixHour end
            if db.showSeconds then
                text1 = string.format("%s%s%s%s%s", newHour, clockSep, minute, clockSep, seconds)
            else
                text1 = string.format("%s%s%s", newHour, clockSep, minute)
            end
            text2 = "pm"
        else
            if fixHour == 0 then newHour = 12 else newHour = fixHour end
            if db.showSeconds then
                text1 = string.format("%s%s%s%s%s", newHour, clockSep, minute, clockSep, seconds)
            else
                text1 = string.format("%s%s%s", newHour, clockSep, minute)
            end
            text2 = "am"
        end
    end
    
    text = string.format("%s%s|r%s%s|r", color1, text1, color2, text2)
    self:UpdateText(self, text)
end
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function SLClock:CreateFrame()
    if self.frame then return else
        self.frame = CreateFrame("Frame", nil, UIParent)
        self.text = self.frame:CreateFontString(nil, "OVERLAY")
        self:UpdateFrame(self)
        SLClockAnchor = self.frame
    end
end

function SLClock:MoveFrame()
    local mdb = SLCore.db.profile
    self.frame:SetPoint(db.justify, db.pos.anchor, db.pos.anchorFrom, db.pos.x, db.pos.y)
    if not mdb.isLocked then self.frame:StartMoving() else end
end

function SLClock:StopFrame()
    local mdb = SLCore.db.profile
    self.frame:StopMovingOrSizing()
    local aP, _, aF, x, y = self.frame:GetPoint()
    local anchor = self.frame:GetParent():GetName()
    local offX, offY = self:TranslateCoords(self, db.justify, aF, x, y)
    db.pos.anchorPoint, db.pos.anchor, db.pos.anchorFrom, db.pos.x, db.pos.y = db.justify, anchor, aF, floor(offX), floor(offY)
end

function SLClock:Print(msg)
    if msg then
        DEFAULT_CHAT_FRAME:AddMessage(msg)
    end
end

-------------------------------------------
--  Options & Defaults
-------------------------------------------

SLClock.defaults = {
    intDelay = 1,
    is24Hour = false,
    showSeconds = false,
    seperator = ":",
    color1 = "White",
    color2 = "White",
    fontFace = "Arial Narrow",
    fontSize = 16,
    isMasterFont = true,
    isMasterFontSize = true,
    isMasterColors = true,
    pos = { anchorPoint = "RIGHT", anchor = "UIParent", anchorFrom = "CENTER", x = 350, y = 0 },
    justify = "RIGHT",
    frameStrata = "BACKGROUND",
    anchToSL = false,
}

SLClock.opts = {
    type = "group",
    args = {
        is24Hour = {
            type = "toggle",
            name = L["Show 24 Hour"],
            desc = L["Show 24 Hour Desc"],
            get = function() return db.is24Hour end,
            set = function() db.is24Hour = not db.is24Hour self:Update() end,
            order = 20,
        },
        showSeconds = {
            type = "toggle",
            name = L["Show Seconds"],
            desc = L["Show Seconds Desc"],
            get = function() return db.showSeconds end,
            set = function() db.showSeconds = not db.showSeconds self:Update() end,
            order = 25,
        },
        clockSep = {
            type = "text",
            name = L["Clock Seperator"],
            desc = L["Clock Seperator Desc"],
            usage = "<Character>",
            get = function() return db.seperator end,
            set = function(v) db.seperator = v self:Update() end,
            order = 35,
        },
        inheritColors = {
            type = "toggle",
            name = L["Inherit Master Colors"],
            desc = L["Inherit Master Colors Desc"],
            get = function() return db.isMasterColors end,
            set = function() db.isMasterColors = not db.isMasterColors self:Update() end,
            order = 40,
        },
        textColor1 = {
            type = "text",
            name = L["First Color"],
            desc = L["First Color Desc"],
            validate = SLCore.Colorlist,
            disabled = function() if db.isMasterColors then return true else return false end end,
            get = function() return db.color1 end,
            set = function(v) db.color1 = v self:Update() end,
            order = 50,
        },
        textColor2 = {
            type = "text",
            name = L["Second Color"],
            desc = L["Second Color Desc"],
            validate = SLCore.Colorlist,
            disabled = function() if db.isMasterColors then return true else return false end end,
            get = function() return db.color2 end,
            set = function(v) db.color2 = v self:Update() end,
            order = 60,
        },
        inheritFont = {
            type = "toggle",
            name = L["Inherit Master Font"],
            desc = L["Inherit Master Font Desc"],
            get = function() return db.isMasterFont end,
            set = function() db.isMasterFont = not db.isMasterFont self:Update() end,
            order = 70,
        },
        inheritFontSize = {
            type = "toggle",
            name = L["Inherit Master Font Size"],
            desc = L["Inherit Master Font Size Desc"],
            get = function() return db.isMasterFontSize end,
            set = function() db.isMasterFontSize = not db.isMasterFontSize self:Update() end,
            order = 80,
        },
        font = {
            type = "text",
            name = L["Font Face"],
            desc = L["Font Face Desc"],
            validate = SML:List("font"),
            disabled = function() if db.isMasterFont then return true else return false end end,
            get = function() return db.fontFace end,
            set = function(v) db.fontFace = v  self:Update() end,
            order = 90,
        },
        fontSize = {
            type = "range",
            name = L["Font Size"],
            desc = L["Font Size Desc"],
            min = 4,
            max = 36,
            step = 1,
            disabled = function() if db.isMasterFontSize then return true else return false end end,
            get = function() return db.fontSize end,
            set = function(v) db.fontSize = v self:Update() end,
            order = 100,
        },
        justify = {
            type = "text",
            name = L["Text Justify"],
            desc = L["Text Justify Desc"],
            validate = SLCore.JustifyTable,
            get = function() return db.justify end,
            set = function(v) db.justify = v self:Update() end,
            order = 110,
        },
        offsetX = {
            type = "range",
            name = L["X Offset"],
            desc = L["X Offset Desc"],
            min = -1000,
            max = 1000,
            step = 1,
            get = function() return db.pos.x end,
            set = function(v) db.pos.x = v self:Update() end,
            order = 120,
        },
        offsetY = {
            type = "range",
            name = L["Y Offset"],
            desc = L["Y Offset Desc"],
            min = -1000,
            max = 1000,
            step = 1,
            get = function() return db.pos.y end,
            set = function(v) db.pos.y = v self:Update() end,
            order = 125,
        },
        frameStrata = {
            type = "text",
            name = L["Frame Strata"],
            desc = L["Frame Strata Desc"],
            validate = SLCore.StrataTable,
            get = function() return db.frameStrata end,
            set = function(v) db.frameStrata = v self:Update() end,
            order = 135,
        },
        parentFrame = {
            type = "text",
            name = L["Parent Frame"],
            desc = L["Parent Frame Desc"],
            usage = L["<Parent Frame>"],
            get = function() return db.pos.anchor end,
            set = function(v) db.pos.anchor = v self:Update() end,
            order = 140,
        },
    },
}